/*
	file:
		ScriptClasses.h
	desc:
		Header file for ScriptClasses.cpp
*/

#ifndef SCRIPTCLASSES_H
#define SCRIPTCLASSES_H

// Includes
#include "GameScripting.h"
#include "MissionHook.h"


//Class definitions
class ScriptScript
{
public:
	ScriptScript();
	~ScriptScript();

	bool ModelAvailable(DWORD dwModel);

	void CreateThread(int StartIP);
	void TerminateThread();
	void NameThread(char ThreadName[8]);
	void Wait(int msTime);
};

class ScriptGame
{
public:
	ScriptGame();
	~ScriptGame();

	bool Fading();

	void Fade(int Time, int InOut);
	void SetMaxWantedLevel(int MaxLevel);
	void SetWastedBustedCheck(bool Check);
	void SetCurrentTime(int Hours, int Minutes);
	void RefreshScreen(float fX, float fY);
	void SetCamera(float fX, float fY, float fZ);
	void SetWeather(int Weather);
	void SelectInterior(int Interior);
	void PlayMusic(int iMusic);
	void TextStyledOneNumber(char Text[8], int iNumber, int iTime, int iStyle);
	void TextStyled(char Text[8], int iTime, int iStyle);
	void TextNow(char Text[8], int iTime, int iStyle);
	void SetWidescreen(bool bWidescreen);
	void SetCameraBehindPlayer();
	void SetCameraOnVehicle(DWORD* pdwVehicle);
	void PointCamera(float fX, float fY, float fZ, int iType);
	void RestoreCameraJumpcut();
	void SetCameraPosition(float fX1, float fY1, float fZ1, float fX2, float fY2, float fZ2);
	void SetWastedSpawnPosition(float fX, float fY, float fZ, float fZAngle);
	void SetBustedSpawnPosition(float fX, float fY, float fZ, float fZAngle);
};

class ScriptPlayer
{
private:
	DWORD m_dwChar;
	DWORD m_dwActor;
public:
	ScriptPlayer(float fX, float fY, float fZ);
	~ScriptPlayer();

	DWORD* GetChar();
	DWORD* GetActor();

	bool NearPoint(float fX, float fY, float fZ, float fRX, float fRY, float fRZ, bool bSphere);
	bool NearPointOnFoot(float fX, float fY, float fZ, float fRX, float fRY, float fRZ, bool bSphere);

	void SetSkin(char Model[8]);
	void Freeze(bool bFrozen);
	void ClearWantedLevel();
	void SetWantedLevel(int iLevel);
	void SetHealth(int iHealth);
	void SetZAngle(float fAngle);
	void GiveWeapon(SCRIPT_MISSION* m_pMission, DWORD dwWeapon, DWORD dwAmmo);
	void GiveMoney(int iMoney);
};

class ScriptActor
{
private:
	DWORD m_dwActor;
	SCRIPT_MISSION* m_pMission;
	bool m_bKeepOnDestroy;
	bool m_bSpawned;
public:
	ScriptActor(SCRIPT_MISSION* pMission, bool bKeepOnDestroy = true);
	~ScriptActor();

	DWORD* GetActor();
	bool Dead();
	bool NearPoint(float fX, float fY, float fZ, float fRX, float fRY, float fRZ, bool bSphere);

	void Spawn(int iPedType, DWORD dwModel, float fX, float fY, float fZ);
	void SpawnInPassengerSeat(DWORD* pdwVehicle, int iPedType, DWORD dwModel, int iSeat);
	void SpawnInDriverSeat(DWORD* pdwVehicle, int iPedType, DWORD dwModel);
	void GiveWeapon(DWORD dwWeapon, DWORD dwAmmo);
	void ResetFlags();
	void SetPedStats(DWORD dwPedStats);
	void SetWander(bool bWander);
	void TieToPlayer(DWORD* pdwPlayer);
	void LeaveVehicle();
	void KillActor(DWORD* pdwActor);
	void KillPlayer(DWORD* pdwPlayer);
};

class ScriptSpecialActor
{
private:
	SCRIPT_MISSION* m_pMission;
	ScriptActor* m_pActor;
	int m_iSlot;
public:
	ScriptSpecialActor(SCRIPT_MISSION* pMission, int iSlot, char Model[8]);
	~ScriptSpecialActor();

	DWORD* GetActor();
	bool Dead();
	bool NearPoint(float fX, float fY, float fZ, float fRX, float fRY, float fRZ, bool bSphere);

	void Spawn(int iPedType, float fX, float fY, float fZ);
	void SpawnInPassengerSeat(DWORD* pdwVehicle, int iPedType, int iSeat);
	void SpawnInDriverSeat(DWORD* pdwVehicle, int iPedType);
	void TieToPlayer(DWORD* pdwPlayer);
	void LeaveVehicle();
	void SetWander(bool bWander);
};

class ScriptVehicle
{
private:
	DWORD m_dwVehicle;
	SCRIPT_MISSION* m_pMission;
	bool m_bKeepOnDestroy;
public:
	ScriptVehicle(SCRIPT_MISSION* pMission, DWORD dwModel, float fX, float fY, float fZ, bool bKeepOnDestroy = true);
	~ScriptVehicle();

	DWORD* GetVehicle();
	int GetHealth();
	bool NearPoint(float fX, float fY, float fZ, float fRX, float fRY, float fRZ, bool bSphere);

	void SetColour(int iPrimary, int iSecondary);
	void SetZAngle(float fAngle);
	void GetRelativeCoordinates(float fX, float fY, float fZ, float* pfX, float* pfY, float* pfZ);
	void DriveToOnRoad(float fX, float fY, float fZ);
	void SetMaxSpeed(float fSpeed);
	void IgnoreTraffic(int iFlag);
	void SetImmuneToNonplayer(bool bImmune);
	void SetDoorStatus(int iStatus);
	void SetSiren(bool bSiren);
	void SetBehaviour(int iBehaviour);
};

class ScriptMarker
{
private:
	DWORD m_dwMarker;
	bool m_bCreated;
public:
	ScriptMarker();
	~ScriptMarker();

	void TieToActor(DWORD* pdwActor, int iSize, int iType);
	void TieToVehicle(DWORD* pdwVehicle, int iSize, int iType);
	void SphereAndIcon(float x, float y, float z, int iIcon);
	void ShowOnRadar(int iSize);
	void SetColor(int iColour);
};

#endif